import logging

from pyamasicp import client
from pyamasicp.commands import Commands, IR_VOL_UP, IR_VOL_DOWN, IR_OK, VERSION_INFO_OTSC_IMPLEMENTATION_VERSION, \
    VERSION_INFO_PLATFORM_LABEL, VERSION_INFO_PLATFORM_VERSION, MODEL_INFO_MODEL_NUMBER, MODEL_INFO_FW_VERSION, \
    MODEL_INFO_BUILD_DATE, IR_CURSOR_UP
import getmac

logging.basicConfig(level=logging.DEBUG)

logger = logging.getLogger(__name__)
logger.setLevel(logging.DEBUG)

logger.debug("MAC: %a" % getmac.get_mac_address(ip='iiyama.home', hostname='iiyama.home'))

cl = client.Client('iiyama.home', mac="DC:62:94:25:02:B3")
cm = Commands(cl)

cm.set_power_state(True)

# c.set_volume(22)

print("Lock state: %s" % cl.send(b'\x01', b'\x1D'))
cm.ir_command(IR_CURSOR_UP)

# logger.info("Current volume level: %s" % (c.get_volume()))
# input_source = c.get_input_source()
# logger.info("Current input source: 0x%02X[%d]" % (input_source[0], input_source[1]))

#c.set_input_source(0x18)


# print("OSC implementation version: %s" % c.get_osc_implementation_version())
# print("Platform label: %s" % c.get_platform_label())
# print("Platform version: %s" % c.get_platform_version())
# print("Model number: %s" % c.get_model_number())
# print("FW version: %s" % c.get_fw_version())
# print("Build date: %s" % c.get_build_date())
# c.get_power_state()


# c.send(b'\x01', b'\x18', b'\x01')

# send(b'\x01', b'\xA2', b'\x01')
# send(b'\x01', b'\xAC', b'\x00\x18\x01\x00')

# send(b'\x01', b'\x44', b'\100\20')


# HDMI1:    0x0d[0]
# HDMI2:    0x06[0]
# HDMI3:    0x0F[0]
# Browser:  0x10[0]
# CMS:      0x11[0]
# Files:    0x13[0]
# Media:    0x16[0]
# PDF:      0x17[0]
# Custom:   0x18[0]



import binascii
import logging
import socket

from pyamasicp.client import Client

CMD_SET_POWER_STATE = b'\x18'
CMD_GET_POWER_STATE = b'\x19'
CMD_SET_VOLUME = b'\x44'
CMD_GET_VOLUME = b'\x45'
CMD_SET_INPUT_SOURCE = b'\xAC'
CMD_GET_INPUT_SOURCE = b'\xAD'
CMD_GET_INFO = b'\xA1'
CMD_GET_VERSION = b'\xA2'
CMD_IR = b'\xDB'

MODEL_INFO_MODEL_NUMBER = b'\x00'
MODEL_INFO_FW_VERSION = b'\x01'
MODEL_INFO_BUILD_DATE = b'\x02'

VERSION_INFO_OTSC_IMPLEMENTATION_VERSION = b'\x00'
VERSION_INFO_PLATFORM_LABEL = b'\x01'
VERSION_INFO_PLATFORM_VERSION = b'\x02'

VAL_POWER_OFF = b'\x01'
VAL_POWER_ON = b'\x02'

IR_POWER = b'\xA0'
IR_MENU = b'\xA1'
IR_INPUT = b'\xA2'
IR_VOL_UP = b'\xA3'
IR_VOL_DOWN = b'\xA4'
IR_MUTE = b'\xA5'
IR_CURSOR_UP = b'\xA6'
IR_CURSOR_DOWN = b'\xA7'
IR_CURSOR_LEFT = b'\xA8'
IR_CURSOR_RIGHT = b'\xA9'
IR_OK = b'\xB1'
IR_RETURN = b'\xB2'
IR_RED = b'\xC1'
IR_GREEN = b'\xC2'
IR_YELLOW = b'\xC3'
IR_BLUE = b'\xC4'
IR_FORMAT = b'\xD1'
IR_INFO = b'\xD2'
IR_BTN_0 = b'\x00'
IR_BTN_1 = b'\x01'
IR_BTN_2 = b'\x02'
IR_BTN_3 = b'\x03'
IR_BTN_4 = b'\x04'
IR_BTN_5 = b'\x05'
IR_BTN_6 = b'\x06'
IR_BTN_7 = b'\x07'
IR_BTN_8 = b'\x08'
IR_BTN_9 = b'\x09'

INPUT_SOURCES = {
    "VIDEO": 0x00,
    "Display Port": 0x01,
    "S-VIDEO": 0x02,
    "COMPONENT": 0x03,
    "VGA": 0x05,
    "HDMI 2": 0x06,
    "Display Port 2": 0x07,
    "USB 2": 0x08,
    "Card DVI-D": 0x09,
    "Display Port 1": 0x0A,
    "Card OPS": 0x0B,
    "USB 1": 0x0C,
    "HDMI 1": 0x0D,
    "DVI-D": 0x0E,
    "HDMI 3": 0x0F,
    "BROWSER": 0x10,
    "SMARTCMS": 0x11,
    "DMS (Digital Media Server)": 0x12,
    "INTERNAL STORAGE": 0x13,
    "Reserved": 0x14,
    "Media Player": 0x16,
    "PDF Player": 0x17,
    "Custom": 0x18,
    "HDMI 4": 0x19,
}


class Commands:

    def __init__(self, client: Client, id=b'\x01'):
        self._logger = logging.getLogger(self.__class__.__qualname__)
        self._client = client
        self._id = id

    def get_power_state(self):
        try:
            result = self._client.send(self._id, CMD_GET_POWER_STATE)
            match result:
                case b'\x01':
                    return False
                case b'\x02':
                    return True
                case None:
                    self._logger.warning("No power state retrieved.")
                    return None
                case _:
                    self._logger.warning("Unknown power state: %s" % binascii.hexlify(result))
                    return None
        except socket.error:
            return None

    def set_power_state(self, state: bool):
        self._client.send(self._id, CMD_SET_POWER_STATE, VAL_POWER_ON if state else VAL_POWER_OFF)

    def get_volume(self):
        result = [b for b in self._client.send(self._id, CMD_GET_VOLUME)]
        result.reverse()
        return result

    def set_volume(self, output_volume=0, volume=0):
        self._client.send(self._id, CMD_SET_VOLUME, bytearray([volume, output_volume, 0, 0]))

    def get_input_source(self):
        return [b for b in self._client.send(self._id, CMD_GET_INPUT_SOURCE)]

    def set_input_source(self, input_type=0, input_number=0, osd_style=0, reserved=0):
        self._client.send(self._id, CMD_SET_INPUT_SOURCE, bytearray([input_type, input_number, osd_style, reserved]))

    def get_osc_implementation_version(self):
        return self._get_string(CMD_GET_VERSION, bytes(VERSION_INFO_OTSC_IMPLEMENTATION_VERSION))

    def get_platform_label(self):
        return self._get_string(CMD_GET_VERSION, bytes(VERSION_INFO_PLATFORM_LABEL))

    def get_platform_version(self):
        return self._get_string(CMD_GET_VERSION, bytes(VERSION_INFO_PLATFORM_VERSION))

    def get_model_number(self):
        return self._get_string(CMD_GET_INFO, bytes(MODEL_INFO_MODEL_NUMBER))

    def get_fw_version(self):
        return self._client.send(CMD_GET_INFO, bytes(MODEL_INFO_FW_VERSION))

    def get_build_date(self):
        return self._client.send(CMD_GET_INFO, bytes(MODEL_INFO_BUILD_DATE))

    def _get_string(self, cmd, data):
        response = self._client.send(self._id, cmd, data)
        return response.decode('utf-8') if response else None

    def ir_command(self, code):
        self._client.send(self._id, CMD_IR, code)


class CommandException(Exception):
    pass



import logging
import socket
import unittest
from unittest.mock import patch

from pyamasicp import commands
from pyamasicp.client import Client
from pyamasicp.commands import Commands

logging.basicConfig(level=logging.DEBUG)

REQUESTS = {
    b'\xa6\x01\x00\x00\x00\x05\x01\x44\x00\x16\xf1': b'\x21\x01\x00\x00\x04\x01\x00\x00\x25',
    b'\xa6\x01\x00\x00\x00\x03\x01\x45\xe0': b'\x21\x01\x00\x00\x05\x01\x45\x16\x16\x61',
    b'\xa6\x01\x00\x00\x00\x03\x01\x19\xbc': b'\x21\x01\x00\x00\x04\x01\x19\x02\x3e',
}


def _mock_remote_call(self, _socket, message):
    return REQUESTS[message]


def _mock_socket(self):
    return socket.socket(socket.AF_UNIX, socket.SOCK_STREAM)


class TestClient(unittest.TestCase):

    def __init__(self, methodName='runTest'):
        super().__init__(methodName)

        # Test data
        self._id = b'\x01'

    @patch.object(Client, '_create_and_connect_socket', side_effect=_mock_socket, autospec=True)
    @patch.object(Client, '_call_remote', side_effect=_mock_remote_call, autospec=True)
    def test_client(self, mocked_create_and_connect_socket, mocked_call_remote):
        # Positive test case
        cl = Client('test.host', mac="00:00:00:00:00:00")
        result = cl.send(self._id, commands.CMD_GET_POWER_STATE)
        self.assertEqual(b'\x02', result)
        mocked_call_remote.assert_called_with(cl)


class TestCommand(unittest.TestCase):

    def __init__(self, methodName='runTest'):
        super().__init__(methodName)

        # Test data
        self._id = b'\x01'

    @patch.object(Client, '_create_and_connect_socket', side_effect=_mock_socket, autospec=True)
    @patch.object(Client, '_call_remote', side_effect=_mock_remote_call, autospec=True)
    def test_get_power_state(self, mocked_create_and_connect_socket, mocked_call_remote):
        # Positive test case
        cmd = Commands(Client('test.host', mac="00:00:00:00:00:00"), self._id)
        result = cmd.get_power_state()
        self.assertEqual(True, result)


if __name__ == "__main__":
    unittest.main()



import logging
import sys

import homeassistant

stdout_handler = logging.StreamHandler(sys.stdout)
stderr_handler = logging.StreamHandler(sys.stderr)

# Set levels for handlers
stdout_handler.setLevel(logging.DEBUG)
stderr_handler.setLevel(logging.WARNING)

# Create formatters and add it to handlers
formatter = logging.Formatter('%(asctime)s - %(name)s - %(levelname)s - %(message)s')
stdout_handler.setFormatter(formatter)
stderr_handler.setFormatter(formatter)

logging.basicConfig(handlers=[stdout_handler, stderr_handler])
